%% Bayesian model selection and averaging
% 
% This script illustrates how to implement fixed-effects and random-
% effects Bayesian model selection (BMS), and how to perform Bayesian 
% model averaging (BMA). Alternatively, the GUI of SPM can be used to 
% click the analyses. The respective matlabbatch can be found under
% "Dynamic causal modeling > Compare". 

% ----------------------------------------------------------------------
% 
% stefan_fraessle@gmx.de
%
% Author: Stefan Fraessle, TNU, UZH & ETHZ - 2018
% Copyright 2018 by Stefan Fraessle <stefan_fraessle@gmx.de>
%
% Licensed under GNU General Public License 3.0 or later.
% Some rights reserved. See COPYING, AUTHORS.
% 
% ---------------------------------------------------------------------


% results array for log model evidence
F = NaN(20,2);

% get the log model evidences (approximated by negative free energies)
for n = 1:20
    if ( n < 10 )
        n_text = ['0' num2str(n)];
    else
        n_text = num2str(n);
    end
    for m = 1:2
        temp = load([pwd '/DCMs/DCM_model' num2str(m) '_' n_text '.mat']);
        F(n,m) = temp.DCM.F;
    end
end

% store the matrix with log model evidences
save([pwd '/DCMs/lme.mat'],'F')


% solution (a): compute subject-sepcific Bayes factors
% -------------------------------------------------------------------------

% compute Bayes factor
BF = exp(F(:,1)-F(:,2));

figure;
col = [0.7 0.7 0.7];
barh(F(:,1)-F(:,2), 'facecolor', col, 'edgecolor', 'none');
ylim([0 size(F,1)+1])
title('log model evidence difference','FontSize',16)
ylabel('subject','FontSize',12)
xlabel('\Delta lme','FontSize',12)
text(-80,20,'Model 2','FontSize',14)
text(10,20,'Model 1','FontSize',14)
box off


% solution (B): compute Group Bayes factor & posterior model probabilities
% -------------------------------------------------------------------------

% compute the Group Bayes factor
sumF	= sum(F,1);
GBF     = exp(sumF - sumF(1));

% compute the posterior model probabilities
sumF    = sumF - max(sumF);
pp      = exp(sumF)./sum(exp(sumF));

% display posterior model probabilities
figure;
col = [0.6 0.6 0.6];
colormap(col);
bar(pp);
xlim([0 3])
set(gca,'xtick',[1 2])
title('Posterior probability','FontSize',16)
xlabel('model','FontSize',12)
ylabel('posterior prob.','FontSize',12)
axis square
box off


% solution (c): perform random effects BMS
% -------------------------------------------------------------------------

% call spm_BMS
[alpha, exp_r, xp, pxp, bor] = spm_BMS(F, 1e6, 1, 0, 1, ones(1,size(F,2)));

% display posterior model probabilities
figure;
col = [0.6 0.6 0.6];
colormap(col);
bar(pxp);
xlim([0 3])
set(gca,'xtick',[1 2])
title('Protected posterior probability','FontSize',16)
xlabel('model','FontSize',12)
ylabel('posterior prob.','FontSize',12)
axis square
box off


% solution (d): compute BMA and evaluate posterior probability
% -------------------------------------------------------------------------

% obtain posterior model probabilities
disp('Obtain posterior model probabilities')
[~,~,~,g_post] = spm_BMS_gibbs(F);

% Models to use in BMA
indx = [1 1];

% Number of samples
nsamp = 1e4;

% Posterior odds ratio for defining Occam's window
odds_ratio = 0.05;

% specify "subj" structure with DCM filenames
for n = 1:20
    if ( n < 10 )
        n_text = ['0' num2str(n)];
    else
        n_text = num2str(n);
    end
    
    for m = 1:2
        temp = load([pwd '/DCMs/DCM_model' num2str(m) '_' n_text '.mat']);
        subj(n).sess.model(m).fname = [pwd '/DCMs/DCM_model' num2str(m) '_' n_text '.mat'];
        subj(n).sess.model(m).F     = temp.DCM.F;
        subj(n).sess.model(m).Ep    = temp.DCM.Ep;
        subj(n).sess.model(m).Cp    = temp.DCM.Cp;
    end
end

% save the model space file
save([pwd '/DCMs/model_space.mat'],'subj')

% compute bayesian model averages
bma = spm_dcm_bma(g_post,indx,subj,nsamp,odds_ratio);

% get posterior mean and covariance
Ep  = bma.mEp;
Cp  = spm_unvec((spm_vec(bma.sEp)).^2,bma.sEp);

% get prior means
model1 = load([pwd '/DCMs/DCM_model1_01.mat']);
model2 = load([pwd '/DCMs/DCM_model2_01.mat']);
DCM.a = model1.DCM.a | model2.DCM.a;
DCM.b = model1.DCM.b | model2.DCM.b;
DCM.c = model1.DCM.c | model2.DCM.c;
DCM.d = zeros(size(DCM.a,1),size(DCM.a,2),0);
[pE,~,~] = spm_dcm_fmri_priors(DCM.a,DCM.b,DCM.c,DCM.d);
T	= full(spm_vec(pE));

% evaluate the posterior probabilities of model parameters
Pp  = spm_unvec(1 - spm_Ncdf(T,abs(spm_vec(Ep)),spm_vec(Cp)),Ep);

% plot the posterior probabilities for A-matrix
figure,
subplot(1,3,1)
imagesc(Pp.A)
colormap('gray')
axis square
xlabel('region(from)')
ylabel('region(to)')
title('A-matrix')
caxis([0 1])

% plot the posterior probabilities for B-matrix (3rd input)
subplot(1,3,2)
imagesc(Pp.B(:,:,3))
colormap('gray')
axis square
title('B-matrix (3rd input)')
xlabel('region(from)')
ylabel('region(to)')
caxis([0 1])

% plot the posterior probabilities for C-matrix
subplot(1,3,3)
imagesc(Pp.C)
colormap('gray')
axis square
title('C-matrix')
xlabel('input')
ylabel('region(to)')
caxis([0 1])
